//*****************************************************************************
//*
//* FILE NAME:  RACFCommandOutput.java
//*
//* IBM Confidential
//*
//*  OCO Source Materials
//*
//*  5647-A01
//*
//*  (C) Copyright IBM Corp. 2000
//*
//*   The source code for this program is not published or otherwise
//*   divested of its trade secrets, irrespective of what has been
//*   deposited with the U.S. Copyright Office.
//*
//*   Status = JMSI619
//*
//*****************************************************************************
//*
//* COMPONENT:  Guenter Dockweiler
//*
//* CLASSES:  RACFCommandOutput (externally documented: no)
//*
//* CHANGE HISTORY:
//*
//*   Change Date     UserID   Description
//*   -----  -------- -------- ------------------------------------------------
//*   c0001  07/01/99 DOCK     initial version
//*   c0002  01/01/00 schawer  -ws for simulation
//*
//*****************************************************************************
package com.ibm.irr.bind;
import java.util.*;

import com.ibm.cim.common.*;
import com.ibm.cim.util.*;

/** RACFCommandOutput is used to analyze the RACF command output according to
  * different criteria.
*/
/** @author G. Dockweiler */

class RACFCommandOutput
{
  private static String ivCopyright = RACFCopyright.IBM_2000;
  private static String CLASS       = "RACFCommandOutput";

  // -------------------------------------------------------------------
  // instance variables;
  // -------------------------------------------------------------------
  private int    ivSAFRC;
  private int    ivRACFRC;
  private int    ivReasonCode;
  private int    ivResultNative;
  private String ivLastCommand;
  private String ivCommandResult;

  private int    ivLocatedLine;

  private static final int MAX_NUM_MESSAGE = 3;

  private static CimExtendedTrace cvTrace = RACFResourceController.getExtendedTrace();

  // -------------------------------------------------------------------
  // constructor
  // -------------------------------------------------------------------
  public RACFCommandOutput ()
  {
    String METHOD = "RACFCommandOutput()";

    if (CimExtendedTrace.cvTrcLvl) { cvTrace.externalCall(null,true,CLASS,METHOD); }

    ivSAFRC         = RACFConstants.RESULT_OK;
    ivRACFRC        = RACFConstants.RESULT_OK;
    ivReasonCode    = RACFConstants.RESULT_OK;
    ivResultNative  = RACFConstants.RESULT_OK;
    ivLastCommand   = "";
    ivCommandResult = null;

    if (CimExtendedTrace.cvTrcLvl) { cvTrace.externalCall(null,false,CLASS,METHOD); }

  }

  /**Actualize command result data
   * @param resultSAFRC is the SAF rc
   * @param resultRACFRC is the RACF rc
   * @param resultReasonCode is the RACF rc of the executed command
   * @param resultNative is rc of the native method
   * @param commandResult is the command output buffer
  */
  // -----------------------------------------------------------------
  // set instance variables with supplied values
  // -----------------------------------------------------------------
  public void setResult(int    resultSAFRC,
                        int    resultRACFRC,
                        int    resultReasonCode,
                        int    resultNative,
                        String lastCommand,
                        String commandResult)
  {
    String METHOD = "setResult()";
    if (CimExtendedTrace.cvTrcLvl) { cvTrace.externalCall("SAF rc: "+resultSAFRC+cvTrace.getIndentString()+
                                     "RACF rc: "+resultRACFRC+cvTrace.getIndentString()+
                                     "RACF reason code: "+resultReasonCode+cvTrace.getIndentString()+
                                     "Native method result: "+resultNative+cvTrace.getIndentString()+
                                     "Command result: "+commandResult.substring(0,Math.min(commandResult.length(),80)),
                                     true,CLASS,METHOD); }

    ivSAFRC         = resultSAFRC;
    ivRACFRC        = resultRACFRC;
    ivReasonCode    = resultReasonCode;
    ivResultNative  = resultNative;
    ivLastCommand   = lastCommand;
    ivCommandResult = commandResult;

    if (CimExtendedTrace.cvTrcLvl) { cvTrace.externalCall(null,false,CLASS,METHOD); }
  }

  /**Get method to retrieve SAF return code
  */
  public int getSAFRC()
  {
    return ivSAFRC;
  }

  /**Get method to retrieve RACF return code
  */
  public int getRACFRC()
  {
    return ivRACFRC;
  }

  /**Get method to retrieve RC of last RACF command, i.e. ivReasonCode
  */
  public int getReasonCode()
  {
    return ivReasonCode;
  }

  /**Get method to retrieve RC of native routine
  */
  public int getResultNative()
  {
    return ivResultNative;
  }

  /**Get method to retrieve command result buffer
  */
  public String getCommandResult()
  {
    return ivCommandResult;
  }

  /**Get method to retrieve RACF error messages in output buffer
  */
  // -----------------------------------------------------------------
  // return error messages having prefix ICH, IKJ or IRR
  // error messages are of interest if
  //  - SAF rc = 8 (or 4)
  //  - RACF rc = 16
  //  - RACF reason code > 0
  //  - the checkOutput...() method did not return the expected result
  // -----------------------------------------------------------------
  public String[] getRACFErrorMessages()
  {
    String METHOD = "getRACFErrorMessages()";
    if (CimExtendedTrace.cvTrcLvl) { cvTrace.externalCall(null,true,CLASS,METHOD); }

    String[] messageList = new String[MAX_NUM_MESSAGE];
    String   tempResult  = ivCommandResult;
    String   oneLine;
    int      nlOffset    = 0;

    for (int line = 0; line < MAX_NUM_MESSAGE; line++)
    {
      messageList[line] = null;

      nlOffset = tempResult.indexOf('\n');
      if (0 <= nlOffset)   // as long as there are lines to inspect
      {
        oneLine    = tempResult.substring(0,nlOffset);
        tempResult = tempResult.substring(nlOffset+1);

        if (oneLine.startsWith("ICH") ||  // these are the lines we
            oneLine.startsWith("IKJ") ||  // are interested in
            oneLine.startsWith("IRR"))
        {
          messageList[line] = oneLine;
        }

      }

    }  // end line loop

    if (CimExtendedTrace.cvTrcLvl) { cvTrace.externalCall("First message: "+messageList[0],false,CLASS,METHOD); }
    return messageList;

  }

  /**Check command output: is active user authorized to issue RACF commands?
   * @return true/false depending on check result
   * @exception RACFInconsistentOutput
  */
  // -----------------------------------------------------------------
  // assumed command: LISTUSER userid
  // - "ATTRIBUTES=" found in output
  //   => "SPECIAL" found in same line
  //      => user is authorized
  //   => "SPECIAL" not found in same line
  //      => user is not authorized
  //  - message "ICH30002I " in line 1
  //    AND command rc 4
  //    => user is not authorized
  //  all other unsuccessful LISTUSER cases => INCONSISTENT_OUTPUT exception
  // -----------------------------------------------------------------
  public boolean userIsAuthorized()
   throws RACFInconsistentOutput
  {
    String METHOD = "userIsAuthorized()";

    if (CimExtendedTrace.cvTrcLvl) { cvTrace.externalCall(null,true,CLASS,METHOD); }

    String  searchArgument;
    boolean result      = false;
    boolean resultFound = false;

    if (false == ivLastCommand.equals("LISTUSER"))
    {
      if (CimExtendedTrace.cvTrcLvl) { cvTrace.diagnostic("Throwing RACFInconsistentOutput exception"); }
      throw new RACFInconsistentOutput("LISTUSER",
                                       ivLastCommand,
                                       this);
    }

    searchArgument = "ATTRIBUTES=";
    if (true == scanOutputBuffer(1,0,searchArgument))
    {
      resultFound = true;
      searchArgument = "SPECIAL";
      if (true == scanOutputBuffer(ivLocatedLine,ivLocatedLine,
                                   searchArgument))
      {
        result      = true;
      }
    }

    searchArgument = "ICH30002I ";
    if ((true == scanOutputBuffer(1,1,searchArgument) &&
         4    == ivReasonCode))
    {
      result      = false;
      resultFound = true;
    }

    if (false == resultFound  &&
        0     <  ivReasonCode)
    {
      if (CimExtendedTrace.cvTrcLvl) { cvTrace.diagnostic("Throwing RACFInconsistentOutput exception"); }
      throw new RACFInconsistentOutput("LISTUSER",
                                       ivLastCommand,
                                       this);
    }

    if (CimExtendedTrace.cvTrcLvl) { cvTrace.externalCall("Check result: "+result,false,CLASS,METHOD); }
    return result;
  }


  /**Check command output: is RACF active?
   * @return true/false depending on check result
  */
  // -----------------------------------------------------------------
  // assumed command: any
  // - SAFRC = 4
  //   => RACF is not active
  // - all other cases
  //   => RACF is active
  // -----------------------------------------------------------------
  public boolean isRACFActive()
  {
    String METHOD = "isRACFActive()";
    if (CimExtendedTrace.cvTrcLvl) { cvTrace.externalCall(null,true,CLASS,METHOD); }

    boolean result = false;

    if (4 == ivSAFRC)
    {
      result = false;
    } else
    {
      result = true;
    }

    if (CimExtendedTrace.cvTrcLvl) { cvTrace.externalCall("Check result: "+result,false,CLASS,METHOD); }
    return result;
  }

  /**Check command output: is RACF subsystem active?
   * @return true/false depending on check result
  */
  // -----------------------------------------------------------------
  // assumed command: any
  // - SAFRC = 8 and RACF RC = 12
  //   => RACF subsystem is not active
  // - all other cases
  //   => RACF subsystem is active
  // -----------------------------------------------------------------
  public boolean isRACFSubsystemActive()
  {
    String METHOD = "isRACFSubsystemActive()";

    if (CimExtendedTrace.cvTrcLvl) { cvTrace.externalCall(null,true,CLASS,METHOD); }

    boolean result = false;

    if (8  == ivSAFRC  &&
        12 == ivRACFRC)
    {
      result = false;
    } else
    {
      result = true;
    }

    if (CimExtendedTrace.cvTrcLvl) { cvTrace.externalCall("Check result: "+result,false,CLASS,METHOD); }
    return result;
  }

  /**Check output of command 'SETROPTS REFRESH RACLIST (class)': is class active?
   * @return true/false depending on check result
  */
  // -----------------------------------------------------------------
  //  - "ICH14041I " in line 1
  //    => class is not active
  //  - all other cases
  //    => class is active
  // -----------------------------------------------------------------
  public boolean isClassActive()
  {
    String METHOD = "isClassActive()";

    if (CimExtendedTrace.cvTrcLvl) { cvTrace.externalCall(null,true,CLASS,METHOD); }

    boolean result         = true;
    String  searchArgument = "ICH14041I ";

    if (true == scanOutputBuffer(1,1,searchArgument))
    {
      result = false;
    }

    if (CimExtendedTrace.cvTrcLvl) { cvTrace.externalCall("Check result: "+result,false,CLASS,METHOD); }
    return result;
  }

  /**Check output of command 'LISTUSER user': is this user defined?
   * @param user is the user ID to be checked
   * @return true/false depending on check result
   * @exception RACFInconsistentOutput
  */
  // -----------------------------------------------------------------
  //  - "USER=user " in line 1
  //    => user exists
  //  - message "ICH30001I " in line 1
  //    AND command rc 4
  //    => user does not exist
  //  - message "IKJ56702I " in line 1
  //    AND command rc 8
  //    => user does not exist (invalid name)
  //  all other cases => INCONSISTENT_OUTPUT exception
  // -----------------------------------------------------------------
  public boolean userExists
   (String user)
   throws RACFInconsistentOutput
  {
    String METHOD = "userExists()";

    if (CimExtendedTrace.cvTrcLvl) { cvTrace.externalCall("User: "+user,true,CLASS,METHOD); }

    String  searchArgument;
    boolean result      = false;
    boolean resultFound = false;

    if (false == ivLastCommand.equals("LISTUSER"))
    {
      if (CimExtendedTrace.cvTrcLvl) { cvTrace.diagnostic("Throwing RACFInconsistentOutput exception"); }
      throw new RACFInconsistentOutput("LISTUSER",
                                       ivLastCommand,
                                       this);
    }

    searchArgument = "USER=" + user + " ";
    if (true == scanOutputBuffer(1,1,searchArgument))
    {
      result      = true;
      resultFound = true;
    }

    searchArgument = "ICH30001I ";
    if ((true == scanOutputBuffer(1,1,searchArgument) &&
         4    == ivReasonCode))
    {
      result      = false;
      resultFound = true;
    }

    searchArgument = "IKJ56702I ";
    if ((true == scanOutputBuffer(1,1,searchArgument) &&
         8    == ivReasonCode))
    {
      result      = false;
      resultFound = true;
    }

    if (false == resultFound)
    {
      if (CimExtendedTrace.cvTrcLvl) { cvTrace.diagnostic("Throwing RACFInconsistentOutput exception"); }
      throw new RACFInconsistentOutput("LISTUSER",
                                       ivLastCommand,
                                       this);
    }

    if (CimExtendedTrace.cvTrcLvl) { cvTrace.externalCall("Check result: "+result,false,CLASS,METHOD); }
    return result;
  }

  /**Check output of command 'LISTGRP group': is this group defined?
   * @param group is the group name to be checked
   * @return true/false depending on check result
   * @exception RACFInconsistentOutput
  */
  // -----------------------------------------------------------------
  //  - "INFORMATION FOR GROUP group " in line 1
  //    => group exists
  //  - message "ICH51003I " in line 1
  //    AND command rc 4
  //    => group does not exist
  //  - message "IKJ56702I " in line 1
  //    AND command rc 8
  //    => group does not exist (invalid name)
  //  all other cases => INCONSISTENT_OUTPUT exception
  // -----------------------------------------------------------------
  public boolean groupExists
   (String group)
   throws RACFInconsistentOutput
  {
    String METHOD = "groupExists()";

    if (CimExtendedTrace.cvTrcLvl) { cvTrace.externalCall("Group: "+group,true,CLASS,METHOD); }

    String  searchArgument;
    boolean result      = false;
    boolean resultFound = false;

    if (false == ivLastCommand.equals("LISTGRP"))
    {
      if (CimExtendedTrace.cvTrcLvl) { cvTrace.diagnostic("Throwing RACFInconsistentOutput exception"); }
      throw new RACFInconsistentOutput("LISTGRP",
                                       ivLastCommand,
                                       this);
    }

    searchArgument = "INFORMATION FOR GROUP " + group;
    if (true == scanOutputBuffer(1,1,searchArgument))
    {
      result      = true;
      resultFound = true;
    }

    searchArgument = "ICH51003I ";
    if ((true == scanOutputBuffer(1,1,searchArgument) &&
         4    == ivReasonCode))
    {
      result      = false;
      resultFound = true;
    }

    searchArgument = "IKJ56702I ";
    if ((true == scanOutputBuffer(1,1,searchArgument) &&
         8    == ivReasonCode))
    {
      result      = false;
      resultFound = true;
    }

    if (false == resultFound)
    {
      if (CimExtendedTrace.cvTrcLvl) { cvTrace.diagnostic("Throwing RACFInconsistentOutput exception"); }
      throw new RACFInconsistentOutput("LISTGRP",
                                       ivLastCommand,
                                       this);
    }

    if (CimExtendedTrace.cvTrcLvl) { cvTrace.externalCall("Check result: "+result,false,CLASS,METHOD); }
    return result;
  }

  /**Check output of command 'RLIST class resource':
    *does specified class exist?
   * @return true/false depending on check result
   * @exception RACFInconsistentOutput
  */
  // -----------------------------------------------------------------
  //  - message "IKJ56702I " in line 1
  //    AND command rc 8
  //    => class does not exist
  //  all other cases => ok
  // -----------------------------------------------------------------
  public boolean classExists ()
   throws RACFInconsistentOutput
  {
    String METHOD = "classExists()";

    if (CimExtendedTrace.cvTrcLvl) { cvTrace.externalCall(null,true,CLASS,METHOD); }

    String  searchArgument;
    boolean result = true;

    if (false == ivLastCommand.equals("RLIST"))
    {
      if (CimExtendedTrace.cvTrcLvl) { cvTrace.diagnostic("Throwing RACFInconsistentOutput exception"); }
      throw new RACFInconsistentOutput("RLIST",
                                       ivLastCommand,
                                       this);
    }

    searchArgument = "IKJ56702I ";
    if ((true == scanOutputBuffer(1,1,searchArgument) &&
         8    == ivReasonCode))
    {
      result      = false;
    }

    if (CimExtendedTrace.cvTrcLvl) { cvTrace.externalCall("Check result: "+result,false,CLASS,METHOD); }
    return result;
  }

  /**Check output of command 'RLIST class resource':
    *is this resource defined in class?
   * @param rclass is the class name of the resource profile to be checked
   * @param resource is the resource profile name to be checked
   * @return true/false depending on check result
   * @exception RACFInconsistentOutput
  */
  // -----------------------------------------------------------------
  //  - "class   resource" in line 3
  //    => resource exists
  //  - "(G)" in line 3
  //    => resource does not exist (only generic profile exists)
  //  - message "ICH13003I " in line 1
  //    AND command rc 4
  //    => resource does not exist
  //  - message "IKJ56702I " in line 1
  //    AND command rc 8
  //    => class does not exist or invalid name => resource does not exist
  //  all other cases => INCONSISTENT_OUTPUT exception
  // -----------------------------------------------------------------
  public boolean resourceExists
   (String rclass,
    String resource)
   throws RACFInconsistentOutput
  {
    String METHOD = "resourceExists()";

    if (CimExtendedTrace.cvTrcLvl) { cvTrace.externalCall("Class: "+rclass+cvTrace.getIndentString()+
                                     "Resource: "+resource,true,CLASS,METHOD); }

    String  searchArgument;
    boolean result      = false;
    boolean resultFound = false;

    if (false == ivLastCommand.equals("RLIST"))
    {
      if (CimExtendedTrace.cvTrcLvl) { cvTrace.diagnostic("Throwing RACFInconsistentOutput exception"); }
      throw new RACFInconsistentOutput("RLIST",
                                       ivLastCommand,
                                       this);
    }

    searchArgument = rclass;
    for (int i=rclass.length()+1; i<=8; i++)
    {
      searchArgument = searchArgument + " ";
    }
    searchArgument = searchArgument + "   " + resource;
    if (true == scanOutputBuffer(3,3,searchArgument))
    {
      result      = true;
      resultFound = true;
    }

    searchArgument = "(G)";
    if (true == scanOutputBuffer(3,3,searchArgument))
    {
      result      = false;
      resultFound = true;
    }

    searchArgument = "ICH13003I ";
    if ((true == scanOutputBuffer(1,1,searchArgument) &&
         4    == ivReasonCode))
    {
      result      = false;
      resultFound = true;
    }

    searchArgument = "IKJ56702I ";
    if ((true == scanOutputBuffer(1,1,searchArgument) &&
         8    == ivReasonCode))
    {
      result      = false;
      resultFound = true;
    }

    if (false == resultFound)
    {
      if (CimExtendedTrace.cvTrcLvl) { cvTrace.diagnostic("Throwing RACFInconsistentOutput exception"); }
      throw new RACFInconsistentOutput("RLIST",
                                       ivLastCommand,
                                       this);
    }

    if (CimExtendedTrace.cvTrcLvl) { cvTrace.externalCall("Check result: "+result,false,CLASS,METHOD); }
    return result;
  }

  /**Check output of command 'RLIST class resource GENERIC':
    *is a generic resource profile defined for this discrete resource profile?
   * @param rclass is the class name of the generic resource profile to be checked
   * @return generic resource profile name if it exists, otherwise null
  */
  // -----------------------------------------------------------------
  //  - extract generic profile name from line 3 (class   resource)
  //  - return extracted name
  // -----------------------------------------------------------------
  public String getGenericResource
   (String rclass)
  {
    String METHOD = "getGenericResource()";

    if (CimExtendedTrace.cvTrcLvl) { cvTrace.externalCall("Class: "+rclass,true,CLASS,METHOD); }

    String  resourceName  = null;
    boolean resourceFound = false;
    int     nlOffset;

    if (false == scanOutputBuffer(1,1,"CLASS      NAME"))
    {
      resourceName  = null;
      resourceFound = true;
    }

    if (false == scanOutputBuffer(3,3,rclass))
    {
      resourceName  = null;
      resourceFound = true;
    }

    if (false == resourceFound)
    {
      resourceName = ivCommandResult;
      nlOffset     = resourceName.indexOf('\n');
      resourceName = resourceName.substring(nlOffset+1);
      nlOffset     = resourceName.indexOf('\n');
      resourceName = resourceName.substring(nlOffset+12);
      nlOffset     = resourceName.indexOf(' ');
      resourceName = resourceName.substring(0,nlOffset);
    }

    if (CimExtendedTrace.cvTrcLvl) { cvTrace.externalCall("Generic resource: "+resourceName,false,CLASS,METHOD); }
    return resourceName;
  }

  /**Check output of command 'LISTDSD DATASET('dataset'): is this dataset defined?
   * @param dataset is the dataset profile name to be checked
   * @return true/false depending on check result
   * @exception RACFInconsistentOutput
  */
  // -----------------------------------------------------------------
  //  - "INFORMATION FOR DATASET dataset" in line 1
  //    => dataset exists
  //  - message "ICH35003I " in line 1
  //    AND command rc 4
  //    => dataset does not exist
  //  - message "IKJ56702I " in line 1
  //    AND command rc 8
  //    => dataset does not exist (invalid name)
  //  all other cases => INCONSISTENT_OUTPUT exception
  // -----------------------------------------------------------------
  public boolean datasetExists
   (String dataset)
   throws RACFInconsistentOutput
  {
    String METHOD = "datasetExists()";

    if (CimExtendedTrace.cvTrcLvl) { cvTrace.externalCall("Dataset: "+dataset,true,CLASS,METHOD); }

    String  searchArgument;
    boolean result      = false;
    boolean resultFound = false;

    if (false == ivLastCommand.equals("LISTDSD"))
    {
      if (CimExtendedTrace.cvTrcLvl) { cvTrace.diagnostic("Throwing RACFInconsistentOutput exception"); }
      throw new RACFInconsistentOutput("LISTDSD",
                                       ivLastCommand,
                                       this);
    }

    searchArgument = "INFORMATION FOR DATASET " + dataset;
    if (true == scanOutputBuffer(1,1,searchArgument))
    {
      result      = true;
      resultFound = true;
    }

    searchArgument = "ICH35003I ";
    if ((true == scanOutputBuffer(1,1,searchArgument) &&
         4    == ivReasonCode))
    {
      result      = false;
      resultFound = true;
    }

    searchArgument = "IKJ56702I ";
    if ((true == scanOutputBuffer(1,1,searchArgument) &&
         8    == ivReasonCode))
    {
      result      = false;
      resultFound = true;
    }

    if (false == resultFound)
    {
      if (CimExtendedTrace.cvTrcLvl) { cvTrace.diagnostic("Throwing RACFInconsistentOutput exception"); }
      throw new RACFInconsistentOutput("LISTDSD",
                                       ivLastCommand,
                                       this);
    }

    if (CimExtendedTrace.cvTrcLvl) { cvTrace.externalCall("Check result: "+result,false,CLASS,METHOD); }
    return result;
  }

  /**Check output of command 'LISTDSD DATASET('dataset') GENERIC':
    *is a generic dataset profile defined for this discrete dataset profile?
   * @return generic dataset profile name if it exists, otherwise null
  */
  // -----------------------------------------------------------------
  //  - extract generic profile name from line 1
  //   ("INFORMATION FOR DATASET "dataset)
  //  - return extracted name
  // -----------------------------------------------------------------
  public String getGenericDataset()
  {
    String METHOD = "getGenericDataset()";

    if (CimExtendedTrace.cvTrcLvl) { cvTrace.externalCall(null,true,CLASS,METHOD); }

    String  datasetName  = null;
    boolean datasetFound = false;
    int     nlOffset;

    if (false == scanOutputBuffer(1,1,"INFORMATION FOR DATASET "))
    {
      datasetName  = null;
      datasetFound = true;
    }

    if (false == datasetFound)
    {
      nlOffset    = "INFORMATION FOR DATASET ".length();
      datasetName = ivCommandResult.substring(nlOffset);
      nlOffset    = datasetName.indexOf(' ');
      datasetName = datasetName.substring(0,nlOffset);
    }

    if (CimExtendedTrace.cvTrcLvl) { cvTrace.externalCall("Generic dataset: "+datasetName,false,CLASS,METHOD); }
    return datasetName;
  }

  /**Check output of command 'LISTUSER user':
    *is this user connected to a certain group?
   * @param group is the group name to be checked
   * @return true/false depending on check result
   * @exception RACFInconsistentOutput
  */
  // -----------------------------------------------------------------
  //  - message "ICH30001I " in line 1
  //  - "GROUP=group " after line containing "LOGON ALLOWED"
  //    => connection exists
  //    AND command rc 4 (user not found for LISTUSER)
  //    => connection does not exist
  //  - message "IKJ56702I " in line 1
  //    AND command rc 8
  //    => connection does not exist (invalid user name)
  //  all other cases => INCONSISTENT_OUTPUT exception
  // -----------------------------------------------------------------
  public boolean connectionExists
   (String group)
   throws RACFInconsistentOutput
  {
    String METHOD = "connectionExists()";

    if (CimExtendedTrace.cvTrcLvl) { cvTrace.externalCall("Group: "+group,true,CLASS,METHOD); }

    String  searchArgument;
    boolean result      = false;
    boolean resultFound = false;

    if (false == ivLastCommand.equals("LISTUSER"))
    {
      if (CimExtendedTrace.cvTrcLvl) { cvTrace.diagnostic("Throwing RACFInconsistentOutput exception"); }
      throw new RACFInconsistentOutput("LISTUSER",
                                       ivLastCommand,
                                       this);
    }

    searchArgument = "LOGON ALLOWED ";
    if (true == scanOutputBuffer(1,0,searchArgument))
    {
      resultFound    = true;
      searchArgument = "GROUP=" + group + " ";
      if (true == scanOutputBuffer(ivLocatedLine+2,0,searchArgument))
      {
        result = true;
      } else
      {
        result = false;
      }
    }

    searchArgument = "ICH30001I ";
    if ((true == scanOutputBuffer(1,1,searchArgument) &&
         4    == ivReasonCode))
    {
      result      = false;
      resultFound = true;
    }

    searchArgument = "IKJ56702I ";
    if ((true == scanOutputBuffer(1,1,searchArgument) &&
         8    == ivReasonCode))
    {
      result      = false;
      resultFound = true;
    }

    if (false == resultFound)
    {
      if (CimExtendedTrace.cvTrcLvl) { cvTrace.diagnostic("Throwing RACFInconsistentOutput exception"); }
      throw new RACFInconsistentOutput("LISTUSER",
                                       ivLastCommand,
                                       this);
    }

    if (CimExtendedTrace.cvTrcLvl) { cvTrace.externalCall("Check result: "+result,false,CLASS,METHOD); }
    return result;
  }

  /**Check output of command 'RLIST class resource AUTHUSER':
   *is this resource in class accessible by a certain user or group?
   *OR
   *Check output of command 'LISTDSD DATASET('dataset') AUTHUSER':
   *is this dataset accessible by a certain user or group?
   * @param userOrGroup is the user ID or group name to be checked
   * @return true/false depending on check result
   * @exception RACFInconsistentOutput
  */
  // -----------------------------------------------------------------
  //  For RLIST output:
  //  - message "ICH13003I " in line 1
  //    AND command rc 4 (resource not found for RLIST)
  //    => access permission does not exist
  //  - message "IKJ56702I " in line 1
  //    AND command rc 8
  //    => class does not exist => access permission does not exist
  //  - "userOrGroup " between "USER      ACCESS" of access list and
  //       "   ID     ACCESS " of conditional access list
  //    => access permission exists
  //  For LISTDSD output:
  //  - message "ICH35003I " in line 1
  //    AND command rc 4 (dataset not found for RLIST)
  //    => access permission does not exist
  //  - "userOrGroup " between "   ID     ACCESS" of access list and
  //       "   ID    ACCESS " of conditional access list
  //    => access permission exists
  //  all other cases => INCONSISTENT_OUTPUT exception
  // -----------------------------------------------------------------
  public boolean accessExists
   (String userOrGroup)
   throws RACFInconsistentOutput
  {
    String METHOD = "accessExists()";

    if (CimExtendedTrace.cvTrcLvl) { cvTrace.externalCall("User or group: "+userOrGroup,true,CLASS,METHOD); }

    String  searchArgument;
    boolean result      = false;
    boolean resultFound = false;

    if (false == ivLastCommand.equals("RLIST")   &&
        false == ivLastCommand.equals("LISTDSD"))
    {
      if (CimExtendedTrace.cvTrcLvl) { cvTrace.diagnostic("Throwing RACFInconsistentOutput exception"); }
      throw new RACFInconsistentOutput("RLIST,LISTDSD",
                                       ivLastCommand,
                                       this);
    }

    if (true == ivLastCommand.equals("RLIST"))
    { searchArgument = "ICH13003I "; }
    else
    { searchArgument = "ICH35003I "; }

    if ((true == scanOutputBuffer(1,1,searchArgument) &&
         4    == ivReasonCode))
    {
      result      = false;
      resultFound = true;
    }

    if (true == ivLastCommand.equals("RLIST"))
    {
      searchArgument = "IKJ56702I ";
      if ((true == scanOutputBuffer(1,1,searchArgument) &&
           8    == ivReasonCode))
      {
        result      = false;
        resultFound = true;
      }
    }

    if (false == resultFound)
    {
      if (true == ivLastCommand.equals("RLIST"))
      { searchArgument = "USER      ACCESS"; }
      else
      { searchArgument = "   ID     ACCESS"; }

      boolean firstSearch = scanOutputBuffer(1,0,searchArgument);
      int     start       = ivLocatedLine + 2;

      if (true == ivLastCommand.equals("RLIST"))
      { searchArgument = "   ID     ACCESS "; }
      else
      { searchArgument = "   ID    ACCESS "; }

      if ((true == firstSearch) &&
          (true == scanOutputBuffer(start,0,searchArgument)))
      {
        searchArgument = userOrGroup + " ";
        if (true == scanOutputBuffer(start,ivLocatedLine-1,searchArgument))
        {
          result      = true;
          resultFound = true;
        } else
        {
          result      = false;
          resultFound = true;
        }
      }
    }

    if (false == resultFound)
    {
      if (CimExtendedTrace.cvTrcLvl) { cvTrace.diagnostic("Throwing RACFInconsistentOutput exception"); }
      throw new RACFInconsistentOutput("RLIST,LISTDSD",
                                       ivLastCommand,
                                       this);
    }

    if (CimExtendedTrace.cvTrcLvl) { cvTrace.externalCall("Check result: "+result,false); }
    return result;
  }

  /**Check output of command 'DELUSER', 'DELGROUP', 'RDELETE', 'DELDSD', 'REMOVE' or 'PERMIT DELETE':
    *did deletion of object fail, because it does not exist anynore?
   * @param objectIndicator identifies the object to be checked, e.g. USER, GROUP, etc
   * @return true/false depending on check result
   * @exception RACFInconsistentOutput
  */
  // -----------------------------------------------------------------
  //  DELUSER       => message = "IKJ56702I" AND allowed reason code = 8
  //  DELGROUP      => message = "IKJ56702I" AND allowed reason code = 8
  //  RDELETE       => message = "ICH12102I" AND allowed reason code = 4
  //                   message = "IKJ56702I" AND allowed reason code = 8
  //  REMOVE        => message = "ICH03002I" AND allowed reason code = 4
  //                   message = "IKJ56702I" AND allowed reason code = 8
  //                   message = "ICH51003I" AND allowed reason code = 8
  //  DELDSD        => message = "ICH09020I" AND allowed reason code = 4
  //  PERMIT DELETE => message = "ICH06002I" AND allowed reason code = 4
  //                   message = "ICH06004I" AND allowed reason code = 8
  //                   message = "IKJ56702I" AND allowed reason code = 8
  //  - message in line 1
  //    AND command rc is allowed reason code
  //    => object does not exist (could not be deleted)
  //  - otherwise
  //    => object exists (delete failed for unexpected reason)
  // -----------------------------------------------------------------
  public boolean deletedObjectDoesNotExist
   (int objectIndicator)
   throws RACFInconsistentOutput
  {
    String METHOD = "deletedObjectDoesNotExist()";

    if (CimExtendedTrace.cvTrcLvl) { cvTrace.externalCall("Object indicator: "+objectIndicator,true,CLASS,METHOD); }

    String  searchArgument;
    boolean result = false;

    if ((false == ivLastCommand.equals("DELUSER"))  &&
        (false == ivLastCommand.equals("DELGROUP")) &&
        (false == ivLastCommand.equals("RDELETE"))  &&
        (false == ivLastCommand.equals("DELDSD"))   &&
        (false == ivLastCommand.equals("REMOVE"))   &&
        (false == ivLastCommand.equals("PERMIT")))
    {
      if (CimExtendedTrace.cvTrcLvl) { cvTrace.diagnostic("Throwing RACFInconsistentOutput exception"); }
      throw new RACFInconsistentOutput(
                     "DELUSER,DELGROUP,RDELETE,DELDSD,REMOVE,PERMIT",
                     ivLastCommand,
                     this);
    }

    switch (objectIndicator)
    {
      case RACFConstants.OBJIND_USER :
      case RACFConstants.OBJIND_GROUP :
      {

        if (true == scanOutputBuffer(1,1,"IKJ56702I ") &&
            8    == ivReasonCode)
        {
          result = true;
        }
        break;

      }

      case RACFConstants.OBJIND_RESOURCE :
      {

        if (true == scanOutputBuffer(1,1,"ICH12102I ") &&
            4    == ivReasonCode)
        {
          result = true;
        }
        if (true == scanOutputBuffer(1,1,"IKJ56702I ") &&
            8    == ivReasonCode)
        {
          result = true;
        }
        break;

      }

      case RACFConstants.OBJIND_DATASET :
      {

        if (true == scanOutputBuffer(1,1,"ICH09020I ") &&
            4    == ivReasonCode)
        {
          result = true;
        }
        break;

      }

      case RACFConstants.OBJIND_CONNECTION :
      {

        if (true == scanOutputBuffer(1,1,"ICH03002I ") &&
            4    == ivReasonCode)
        {
          result = true;
        }
        if (true == scanOutputBuffer(1,1,"IKJ56702I ") &&
            8    == ivReasonCode)
        {
          result = true;
        }
        if (true == scanOutputBuffer(1,1,"ICH51003I ") &&
            8    == ivReasonCode)
        {
          result = true;
        }
        break;

      }

      case RACFConstants.OBJIND_ACCESS :
      {

        if (true == scanOutputBuffer(1,1,"ICH06002I ") &&
            4    == ivReasonCode)
        {
          result = true;
        }
        if (true == scanOutputBuffer(1,1,"ICH06004I ") &&
            8    == ivReasonCode)
        {
          result = true;
        }
        if (true == scanOutputBuffer(1,1,"IKJ56702I ") &&
            8    == ivReasonCode)
        {
          result = true;
        }
        break;

      }

      default:  // no other objects allowed for this check
      {
        if (CimExtendedTrace.cvTrcLvl) { cvTrace.diagnostic("Throwing RACFInconsistentOutput exception"); }
        throw new RACFInconsistentOutput(
                       "DELUSER,DELGROUP,RDELETE,DELDSD,REMOVE,PERMIT",
                       ivLastCommand,
                       this);
      }

    }

    if (CimExtendedTrace.cvTrcLvl) { cvTrace.externalCall("Check result: "+result,false,CLASS,METHOD); }
    return result;
  }

  /**Search for argument in command result buffer
  */
  // -----------------------------------------------------------------
  // - start search in 'fromLine', end search in 'toLine'
  // - set iLocatedLine to line where argument is found
  // - if argument is found, return true, otherwise false
  // -----------------------------------------------------------------
  private boolean scanOutputBuffer
   (int    fromLine,
    int    toLine,
    String argument)
  {
    String METHOD = "scanOutputBuffer()";

    if (CimExtendedTrace.cvTrcLvl) { cvTrace.internalCall("From line: "+fromLine+cvTrace.getIndentString()+
                                     "To line: "+toLine+cvTrace.getIndentString()+
                                     "Argument: "+argument,true,CLASS,METHOD); }
/* -ws [c0002]
    boolean result     = false;
    int     nlOffset   = 0;
    String  tempResult = ivCommandResult;
    String  oneLine    = "";

    ivLocatedLine = 0;

    if (0 == toLine) toLine = Integer.MAX_VALUE; // inspect all lines

    for (int line = 1; line <= toLine; line++)
    {
      int remainingLength = tempResult.length() + oneLine.length();
      if (nlOffset < remainingLength)
      {

        nlOffset   = tempResult.indexOf('\n');
        oneLine    = tempResult.substring(0,nlOffset);
        tempResult = tempResult.substring(nlOffset+1);

        if (line >= fromLine)
        {
          if (oneLine.indexOf(argument) >= 0)
          {
            result        = true;
            ivLocatedLine = line;
            break;
          }
        }

      } else break;
    }

    if (CimExtendedTrace.cvTrcLvl) { cvTrace.verboseInfo("Located line: "+ivLocatedLine); }
    if (CimExtendedTrace.cvTrcLvl) { cvTrace.internalCall("Scan result: "+result,false); }
    return result;
*/
    return true; // -ws [c0002]
  }

}
