//*****************************************************************************
//*
//* FILE NAME:  RACFPrimeData.java
//*
//* IBM Confidential
//*
//*  OCO Source Materials
//*
//*  5647-A01
//*
//*  (C) Copyright IBM Corp. 2000
//*
//*   The source code for this program is not published or otherwise
//*   divested of its trade secrets, irrespective of what has been
//*   deposited with the U.S. Copyright Office.
//*
//*   Status = JMSI619
//*
//*****************************************************************************
//*
//* COMPONENT:  Guenter Dockweiler
//*
//* CLASSES:  RACFPrimeData (externally documented: no)
//*
//* CHANGE HISTORY:
//*
//*   Change Date     UserID   Description
//*   -----  -------- -------- ------------------------------------------------
//*   c0001  08/04/99 DOCK     initial version
//*
//*****************************************************************************
package com.ibm.irr.bind;

import java.io.*;
import java.util.*;

import com.ibm.cim.util.*;
import com.ibm.cim.common.*;
import com.ibm.cim.directory.*;

/** RACF related refresh data
*/
/** @author G. Dockweiler */

class RACFPrimeData
{
  // Structure of RACF public data
  //
  //  +-------------------------------------------------------------------------------+
  //  |activeRACF: BOOLEAN                                                            |
  //  +-------------------------------------------------------------------------------+
  //  |systemProfileSharing                                                           |
  //  |TABLE +-----------------------------------+------------------------------------+
  //  |      |systemProfileSetName: STRING       |systemProfiles: STRING              |
  //  |      +-----------------------------------+------------------------------------+
  //  |      |             ...                   |              ...                   |
  //  |      +----------+------------+-----------+------------------------------------+
  //  +-------------------------------------------------------------------------------+
  //  |classList:                                                                     |
  //  |TABLE +----------+------------+------------------------------------------------+
  //  |      |className:|classActive:|profileList:                                    |
  //  |      |  STRING  |  BOOLEAN   |TABLE +------------+------+---------------------+
  //  |      |          |            |      |profileName:|uacc: |acessList:           |
  //  |      |          |            |      |   STRING   |STRING|TABLE +------+-------+
  //  |      |          |            |      |            |      |      |user: |access:|
  //  |      |          |            |      |            |      |      +------+-------+
  //  |      |          |            |      |            |      |      |STRING|STRING |
  //  |      |          |            |      |            |      |      | ...  | ...   |
  //  |      |          |            |      |            |      |      +------+-------+
  //  |      |          |            |      |    ...     | ...  |          ...        |
  //  |      |          |            |      +------------+------+---------------------+
  //  |      |   ...    |    ...     |                          ...                   |
  //  |      +----------+------------+------------------------------------------------+
  //  +-------------------------------------------------------------------------------+

  private static String    ivCopyright = RACFCopyright.IBM_2000;
  private static String    CLASS       = "RACFPrimeData";

  private        String             ivPersistentConfiguration;
  private        CimDirectoryAccess ivLDAPAccess;
  private        CimDirectoryData   ivRefreshData;
  private        CimTable           ivClassTable;
  private        Vector             ivAffectedObjects;

  private static String[]           ivClasses = {"FACILITY",
                                                 "LOGSTRM" };
  private static String[]           ivFacilityClasses = {"MVSADMIN.XCF.ARM",
                                                         "MVSADMIN.XCF.CFRM",
                                                         "MVSADMIN.XCF.SFM",
                                                         "MVSADMIN.LOGR" };
  private static String[]           ivSharingTableColumns = {RACFConstants.REFRESH_SHARING_SETNAME,
                                                             RACFConstants.REFRESH_SHARING_PROFILES };
  private static String[]           ivClassTableColumns = {RACFConstants.REFRESH_CLASS_NAME,
                                                           RACFConstants.REFRESH_CLASS_ACTIVE,
                                                           RACFConstants.REFRESH_PROFILE_TABLE };
  private static String[]           ivProfileTableColumns = {RACFConstants.REFRESH_PROFILE_NAME,
                                                             RACFConstants.REFRESH_PROFILE_UACC,
                                                             RACFConstants.REFRESH_ACCESS_TABLE };
  private static String[]           ivAccessTableColumns = {RACFConstants.REFRESH_ACCESS_USER,
                                                            RACFConstants.REFRESH_ACCESS_ACCESS };
  private        RACFCommandBuilder ivCommandBuilder;
  private        RACFResourceAccess ivResourceAccess;
  private        RACFCommandOutput  ivCommandOutput;

  private static CimExtendedTrace cvTrace = RACFResourceController.getExtendedTrace();

  /**Load RACF DDML XML to prepare access to refresh data
   * @param handleLDAPAccess is the LDAP directory access object
   * @param persistentConfiguration is the persistent configuration
   * @exception CimException
  */
  // -------------------------------------------------------------------
  // constructor
  // -------------------------------------------------------------------
  public RACFPrimeData
   (CimDirectoryAccess handleLDAPAccess,
    String             persistentConfiguration,
    RACFCommandBuilder commandBuilder,
    RACFResourceAccess resourceAccess,
    RACFCommandOutput  commandOutput)
    throws CimException
  {
    String METHOD ="RACFPrimeData()";

    if (CimExtendedTrace.cvTrcLvl) { cvTrace.externalCall("Persistent configuration: "+persistentConfiguration,true,CLASS,METHOD); }

    ivLDAPAccess              = handleLDAPAccess;
    ivPersistentConfiguration = persistentConfiguration;
    ivCommandBuilder          = commandBuilder;
    ivResourceAccess          = resourceAccess;
    ivCommandOutput           = commandOutput;
    ivAffectedObjects         = new Vector();
    ivRefreshData             = new CimDirectoryData();

    // read current version of public data
    if (false == ivRefreshData.readFromDirectory(ivLDAPAccess,ivPersistentConfiguration))
    {
      // if public data area does not already exist,
      //    make sure that a default is set for the RACF active indicator
      //    and an empty table for system profile sharing is defined
      //    (getSharedDatabaseID() and refresh dialog assume that it is there)
      ivRefreshData.setBoolean(RACFConstants.REFRESH_RACF_ACTIVE,false);

      CimTable sharingTable = new CimTable(ivSharingTableColumns);
      sharingTable.setName(RACFConstants.REFRESH_SHARING_TABLE);
      ivRefreshData.setTable(RACFConstants.REFRESH_SHARING_TABLE,sharingTable);

      ivRefreshData.writeToDirectory(ivLDAPAccess,
                                     ivPersistentConfiguration);
    }

    if (CimExtendedTrace.cvTrcLvl) { cvTrace.externalCall(null,false,CLASS,METHOD); }
  }

  /**Get name of system profile set containing the driving system profile
   *(system profile sets defined by user in refresh dialog)
   * @return name of system profile set
   * @exception CimException
  */
  // -------------------------------------------------------------------
  public String getSharedDatabaseID(String drivingProfileDN)
    throws CimException
  {
    String METHOD ="getSharedDatabaseID()";
    if (CimExtendedTrace.cvTrcLvl) { cvTrace.externalCall("Driving system profile DN: "+drivingProfileDN,true,CLASS,METHOD); }

    // !!! driving profile name must be determined from driving profile DN
    String drivingProfile = drivingProfileDN;

    String profileSet = RACFConstants.SECURITY_DEFAULT_GROUP;
    // !!! must be determined from refresh data:
    // !!! profileSet = systemProfileSetName, where
    // !!!   PublicData.systemProfileSharing.systemProfileSetName.systemProfileList
    // !!!   contains driving image name as systemProfileName
//  CimTable sharingTable = ivRefreshData.getTable(RACFConstants.REFRESH_SHARING_TABLE);
//  for (int i=0; i<sharingTable.getRowNumber(); i++)
//  {
//    Object[]        row        = sharingTable.getRow(i);
//    StringTokenizer profiles   = new StringTokenizer((String)row[1]);
//    while (profiles.hasMoreTokens())
//    {
//      if (drivingProfile.equals(profiles.nextToken()))
//      { profileSet = (String)row[0]; }
//    }
//  }

    if (CimExtendedTrace.cvTrcLvl) { cvTrace.externalCall("System profile set: "+profileSet,false,CLASS,METHOD); }
    return profileSet;
  }

  /**Get value of 'RACF active' indicator
   * @return value of 'RACF active' indicator in public area
   * @exception CimException
  */
  // -------------------------------------------------------------------
  public boolean getRACFActive()
    throws CimException
  {
    String METHOD ="getRACFActive()";

    if (CimExtendedTrace.cvTrcLvl) { cvTrace.externalCall(null,true,CLASS,METHOD); }

    boolean result;

    result = ivRefreshData.getBoolean(RACFConstants.REFRESH_RACF_ACTIVE);

    if (CimExtendedTrace.cvTrcLvl) { cvTrace.externalCall("Result: "+result,false,CLASS,METHOD); }
    return result;
  }

  /**Set value of 'RACF active' indicator in public area
   * @param value of 'RACF active' indicator
   * @exception CimException
  */
  // -------------------------------------------------------------------
  public void setRACFActive(boolean activeRACF)
    throws CimException
  {
    String METHOD ="setRACFActive()";

    if (CimExtendedTrace.cvTrcLvl) { cvTrace.externalCall("Active RACF :"+activeRACF,true,CLASS,METHOD); }

    ivRefreshData.setBoolean(RACFConstants.REFRESH_RACF_ACTIVE,activeRACF);
    ivRefreshData.writeToDirectory(ivLDAPAccess,
                                   ivPersistentConfiguration);

    if (CimExtendedTrace.cvTrcLvl) { cvTrace.externalCall(null,false,CLASS,METHOD); }
  }

  /**Set all values of class refresh data (class active, list of resource profiles,
   * access list (user, access right) for each profile)
   * @exception CimException RACFInconsistentOutput RACFInternalError
  */
  // -------------------------------------------------------------------
  public void setRACFClassData()
    throws CimException,RACFInconsistentOutput,RACFInternalError
  {
    String METHOD ="setRACFClassData()";

    if (CimExtendedTrace.cvTrcLvl) { cvTrace.externalCall(null,true,CLASS,METHOD); }

    // always re-build everything from scratch
    ivClassTable      = new CimTable(ivClassTableColumns);
    ivClassTable.setName(RACFConstants.REFRESH_CLASS_TABLE);

    // fill class table with current data
 /* c0100
    for (int i=0; i<ivClasses.length; i++)
    { setClassData(ivClasses[i]); }
 */
    // set newly built class table in public data
    ivRefreshData.setTable(RACFConstants.REFRESH_CLASS_TABLE,ivClassTable);
    ivRefreshData.writeToDirectory(ivLDAPAccess,
                                   ivPersistentConfiguration);

    if (CimExtendedTrace.cvTrcLvl) { cvTrace.externalCall(null,false,CLASS,METHOD); }
  }

  /**Set all values of class refresh data (class active, list of resource profiles,
   * access list (user, access right) for each profile)
   * @param class is the name of the RACF class for which data are set
   * @exception CimException RACFInconsistentOutput RACFInternalError
  */
  // -------------------------------------------------------------------
  private void setClassData(String className)
    throws CimException,RACFInconsistentOutput,RACFInternalError
  {
    String METHOD ="setClassData()";

    boolean classActive;

    if (CimExtendedTrace.cvTrcLvl) { cvTrace.externalCall("Class name :"+className,true,CLASS,METHOD); }
    if (CimExtendedTrace.cvTrcLvl) { cvTrace.information("Retrieving refresh data class "+className,CLASS,METHOD); }

    String  command       = ivCommandBuilder.specialOperation("REFRESH RACLIST(" + className + ")");
    boolean commandResult = ivResourceAccess.issueCommand(command,ivCommandOutput);
    classActive           = ivCommandOutput.isClassActive();

    if (CimExtendedTrace.cvTrcLvl) { cvTrace.information("... class active: "+classActive,CLASS,METHOD); }

    // build new row for class table ...
    // ... 1st class table column is className
    // ... 2nd class table column is classActive
    Object[] classTableRow = new Object[3];
    classTableRow[0] = className;
    classTableRow[1] = new Boolean(classActive);

    // create profile table and get all profiles for the requested class
    CimTable profileTable = new CimTable(ivProfileTableColumns);
    profileTable.setName(RACFConstants.REFRESH_PROFILE_TABLE);
    Vector profileList = getProfileList(className);

    // loop through vector containing the profile names
    Enumeration ep = profileList.elements();
    while (ep.hasMoreElements())
    {
      // build new row for list profile table ...
      Object[] profileTableRow = new Object[3];

      String profileName = (String)ep.nextElement();
      if (CimExtendedTrace.cvTrcLvl) { cvTrace.information("... profile name: "+profileName,CLASS,METHOD); }

      // save affected object 'class profile'
      ivAffectedObjects.addElement(className+" "+profileName);

      // ... 1st profile table column is profile name
      profileTableRow[0] = profileName;

      // get access list for current profile
      Vector accessList = getAccessList(className,profileName);
      Enumeration ea = accessList.elements();
      String uacc    = (String)ea.nextElement();
      if (CimExtendedTrace.cvTrcLvl) { cvTrace.information("...... UACC: "+uacc,CLASS,METHOD); }

      // ... 2nd profile table column is uacc
      profileTableRow[1] = uacc;

      // create access list table
      CimTable accessTable = new CimTable(ivAccessTableColumns);
      accessTable.setName(RACFConstants.REFRESH_ACCESS_TABLE);

      while (ea.hasMoreElements())
      {
        // build new row for access list table ...
        Object[] accessTableRow = new Object[2];

        StringTokenizer accessLine = new StringTokenizer((String)ea.nextElement());
        String user   = accessLine.nextToken();
        String access = accessLine.nextToken();
        if (CimExtendedTrace.cvTrcLvl) { cvTrace.information("...... access list (user):   "+user,CLASS,METHOD); }
        if (CimExtendedTrace.cvTrcLvl) { cvTrace.information("...... access list (access): "+access,CLASS,METHOD); }

        // ... 1st acccess list table column is user
        accessTableRow[0] = user;
        // ... 2nd acccess list table column is access
        accessTableRow[1] = access;
        // insert complete new row (user,access) into list accessList
        accessTable.addRow(accessTableRow);
      }

      // ... 3rd profile table column is access table
      profileTableRow[2] = accessTable;
      // insert complete new row (profile,uacc,accessList) into profile table
      profileTable.addRow(profileTableRow);
    }

    // ... 3rd class table column is profile table
    classTableRow[2] = profileTable;
    // insert complete new row (className,classActive,profileList) into class table
    ivClassTable.addRow(classTableRow);

    if (CimExtendedTrace.cvTrcLvl) { cvTrace.information("Refresh data complete for class "+className,CLASS,METHOD); }
    if (CimExtendedTrace.cvTrcLvl) { cvTrace.externalCall(null,false,CLASS,METHOD); }
  }

  /**Checks whether a certain object is part of the data which is subject of refresh
   * @param objectType is the type of the object to be checked (defined in {@link RACFConstants.html})
   * @param objectPartI is the first part of the object, e.g. class name for resource profile object
   * @param objectPartII is the second part of the object, e.g. profile name for resource profile object
   * @param objectPartIII is the third part of the object, e.g. user/group for access object
   * @return 'true' if object is subject of refresh, otherwise 'false'
  */
  // -------------------------------------------------------------------
  public boolean isObjectRefreshable (String objectType, String objectPartI, String objectPartII, String objectPartIII)
  {
    String METHOD ="isObjectAffected()";
    if (CimExtendedTrace.cvTrcLvl) { cvTrace.internalCall("Object type :"+objectType+cvTrace.getIndentString()+
                                     "Object part I :"+objectPartI+cvTrace.getIndentString()+
                                     "Object part II :"+objectPartI+cvTrace.getIndentString()+
                                     "Object part III :"+objectPartII,true,CLASS,METHOD); }

    boolean result         = false;
    String  affectedObject = "";

    if (objectType.equals(RACFConstants.OBJ_RESOURCE))
    { affectedObject = objectPartI + " " + objectPartII; }  // 'class profile'

    Enumeration e = ivAffectedObjects.elements();
    while (e.hasMoreElements())
    {
      if (affectedObject.equals((String)e.nextElement()))
      { result = true; }
    }

    if (CimExtendedTrace.cvTrcLvl) { cvTrace.externalCall("Result: "+result,false,CLASS,METHOD); }
    return result;
  }

  // -------------------------------------------------------------------
  private Vector getProfileList(String rclass)
    throws RACFInconsistentOutput,RACFInternalError
  {
    String METHOD ="getProfileList()";
    if (CimExtendedTrace.cvTrcLvl) { cvTrace.internalCall("Class :"+rclass,true,CLASS,METHOD); }

    Vector  profileList = new Vector();

    if (rclass.equals("FACILITY"))
    {
      for (int i=0; i<ivFacilityClasses.length; i++)
      { profileList.addElement(ivFacilityClasses[i]); }
    }
    else
    {
      String  command       = ivCommandBuilder.checkResourceExistence(rclass,"*");
      boolean commandResult = ivResourceAccess.issueCommand(command,ivCommandOutput);
/* c0100
      profileList           = ivCommandOutput.getProfileList();
*/
    }

    if (CimExtendedTrace.cvTrcLvl) { cvTrace.internalCall(null,false,CLASS,METHOD); }
    return profileList;
  }

  // -------------------------------------------------------------------
  private Vector getAccessList(String rclass, String profile)
    throws RACFInconsistentOutput,RACFInternalError
  {
    String METHOD ="getAccessList()";
    if (CimExtendedTrace.cvTrcLvl) { cvTrace.internalCall("Class: "+rclass+cvTrace.getIndentString()+
                                     "Profile: "+profile,true,CLASS,METHOD); }

    Vector  accessList = new Vector();

    String  command       = ivCommandBuilder.checkAccessExistence(rclass,profile);
    boolean commandResult = ivResourceAccess.issueCommand(command,ivCommandOutput);

/* c0100
    accessList            = ivCommandOutput.getAccessList();
*/
    if (CimExtendedTrace.cvTrcLvl) { cvTrace.internalCall(null,false,CLASS,METHOD); }
    return accessList;
  }

}
